#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-03-31
#
# Monitors Cisco WLC CPU utilization
#
# tested with WLC2504
#
# 2016-03-31: initial release
# 2016-07-02: fixed crit/warn to >=
# 2018-01-15: changed scan function, added wato plugin
# 2021-07-15: rewritten for CMK 2.0
# 2023-06-07: moved gui files to ~/local/lib/chek_mk/gui/plugins/...
#

from typing import Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    SNMPTree,
    contains,
    render,
    check_levels,
)


def parse_cisco_wlc_cpu(string_table: StringTable) -> Optional[int]:
    try:
        return int(string_table[0][0])
    except IndexError:
        pass


def discovery_cisco_wlc_cpu(section: int) -> DiscoveryResult:
    yield Service()


def check_cisco_wlc_cpu(params, section: int) -> CheckResult:
    yield from check_levels(
        value=section,
        label='Total CPU',
        levels_upper=params['levels_upper'],
        render_func=render.percent,
        metric_name='util',
        boundaries=(0, 100),
    )


register.snmp_section(
    name='cisco_wlc_cpu',
    parse_function=parse_cisco_wlc_cpu,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.14179.1.1.5',  # AIRESPACE-SWITCHING-MIB::agentResourceInfoGroup
        oids=[
            '1',  # agentCurrentCPUUtilization
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'Cisco Controller'),  # sysDescr
)

register.check_plugin(
    name='cisco_wlc_cpu',
    service_name='CPU utilization',
    discovery_function=discovery_cisco_wlc_cpu,
    check_function=check_cisco_wlc_cpu,
    check_default_parameters={'levels_upper': (70, 80)},
    check_ruleset_name='cisco_wlc_cpu',
)
